@@ -2,7 +2,7 @@ power.exact.test <-
 function(p1, p2, n1, n2, alternative=c("two.sided", "less", "greater"), alpha=0.05,
                              npNumbers=100, np.interval=FALSE, beta=0.001,
                              method=c("z-pooled", "z-unpooled", "boschloo", "santner and snell", "csm", "csm approximate", "fisher", "chisq", "yates chisq"),
-                             ref.pvalue=TRUE, simulation=FALSE, nsim = 100, delta=0, convexity=TRUE){
+                             tsmethod=c("square", "central"), ref.pvalue=TRUE, simulation=FALSE, nsim = 100, delta=0, convexity=TRUE){
   
   stopifnot(is.logical(np.interval) && is.logical(ref.pvalue) && is.logical(simulation) && is.logical(convexity))
   if (alpha < 0 || alpha >= 0.5) { stop("To improve code efficiency, alpha must be between 0 and 0.5") }
@@ -12,18 +12,20 @@ function(p1, p2, n1, n2, alternative=c("two.sided", "less", "greater"), alpha=0.
   
   alternative <- match.arg(tolower(alternative), c("two.sided", "less", "greater"))
   
-  #Sometimes Z-pooled is called score and Z-unpooled is called wald statistic
+  # The Z-pooled statistic that calculates the variance using MLE, which is the pooled variance if delta=0.
+  # The Z-pooled statistic is also (perhaps better) known as the Score statistic
+  # The classic z-pooled statistic is not performed as the performance is inferior when delta != 0
   if (length(method)==1 && tolower(method)=="score") { method <- "z-pooled" }
-  if (length(method)==1 && tolower(method)=="wald") { method <- "z-unpooled" }
+  #if (length(method)==1 && tolower(method)=="wald") { method <- "z-unpooled" }
   method <- match.arg(tolower(method), c("z-pooled", "z-unpooled", "boschloo", "santner and snell",
                                          "csm", "csm approximate", "fisher", "chisq", "yates chisq"))
   
   if (p1 < 0 || p1 > 1 || p2 < 0 || p2 > 1) { stop("Probabilities must be between 0 and 1") }
   if (n1 <= 0 || n2 <= 0) { stop("Fixed sample sizes must be greater than 0") }
   
-  if (delta != 0 && !(method %in% c("z-pooled", "csm"))) {
-    stop("Delta != 0 only works for Z-pooled and CSM tests")
-  }
+  # if (delta != 0 && !(method %in% c("z-pooled", "csm"))) {
+  #   stop("Delta != 0 only works for Z-pooled and CSM tests")
+  # }
   
   if (method %in% c("csm", "csm approximate", "fisher", "chisq", "yates chisq") && np.interval) {
     warning("Interval of nuisance parameter cannot be used with CSM, fisher, or chi-square test; np.interval changed to FALSE")
@@ -32,8 +34,9 @@ function(p1, p2, n1, n2, alternative=c("two.sided", "less", "greater"), alpha=0.
   
   # Consider all tables:
   if (!simulation) {
-    rejectRegion <- exact.reject.region(n1 = n1, n2 = n2, alternative=alternative, alpha=alpha, npNumbers=npNumbers, np.interval=np.interval, beta=beta,
-                                        method=method, ref.pvalue=ref.pvalue, delta=delta, convexity=convexity)
+    rejectRegion <- exact.reject.region(n1 = n1, n2 = n2, alternative=alternative, alpha=alpha,
+                                        npNumbers=npNumbers, np.interval=np.interval, beta=beta,
+                                        method=method, tsmethod=tsmethod, ref.pvalue=ref.pvalue, delta=delta, convexity=convexity)
     prob <- dbinom(0:n1, n1, p1) %*% t(dbinom(0:n2, n2, p2))
     power <- sum(prob[as.logical(rejectRegion)])
     
@@ -48,13 +51,13 @@ function(p1, p2, n1, n2, alternative=c("two.sided", "less", "greater"), alpha=0.
           (alternative=="less" && (randTables[i,1]/n1 - randTables[i,3]/n2) < delta) || 
           (alternative=="two.sided" && delta==0 && randTables[i,1]/n1 != randTables[i,3]/n2)) {
         if (method=="fisher") {
-          moreExtreme[i] <- (fisher.2x2(matrix(randTables[i,], 2, 2, byrow=TRUE), alternative=alternative) <= alpha)
+          moreExtreme[i] <- (fisher.2x2(matrix(randTables[i,], 2, 2, byrow=TRUE), alternative=alternative)[3] <= alpha)
         } else if (method %in% c("chisq", "yates chisq")) {
           moreExtreme[i] <- suppressWarnings(prop.test(matrix(randTables[i,], 2, 2, byrow=TRUE),
                                                        alternative=alternative, correct=(method == "yates chisq"))$p.value <= alpha)
         } else {
           moreExtreme[i] <- (exact.test(matrix(randTables[i,], 2, 2, byrow=TRUE), npNumbers=npNumbers, alternative=alternative,
-                                        np.interval=np.interval, beta=beta, method=method, to.plot=FALSE, ref.pvalue=ref.pvalue, delta=delta, reject.alpha=alpha))
+                                        np.interval=np.interval, beta=beta, method=method, tsmethod=tsmethod, to.plot=FALSE, ref.pvalue=ref.pvalue, delta=delta, reject.alpha=alpha))
         }
       } else { moreExtreme[i] <- FALSE }
     }
